package jp.co.kayo.android.localplayer.ds.boxnet;

import java.util.Calendar;

import jp.co.kayo.android.localplayer.ds.boxnet.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.ds.boxnet.MediaConsts.AudioArtist;
import jp.co.kayo.android.localplayer.ds.boxnet.MediaConsts.AudioMedia;

import com.box.androidlib.Utils.BoxConfig;

import android.content.ContentValues;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.net.Uri;
import android.provider.BaseColumns;

public class BoxnetHelper {
    private SharedPreferences mPrefs;
    private String mAuthToken;
    private Context mContext;

    public BoxnetHelper(Context context) {
        mContext = context;
        mPrefs = context.getSharedPreferences(SystemConsts.PREFS_FILE_NAME, 0);
        mAuthToken = mPrefs.getString(SystemConsts.PREFS_KEY_AUTH_TOKEN, null);
    }

    public boolean isLoggdIn() {
        return true;
    }
    
    public String getAuthToken(){
        return mAuthToken;
    }
    
    public Context getContext(){
        return mContext;
    }

    public String getAuthrizedURL(String fileId) {
        final Uri.Builder builder = new Uri.Builder();
        builder.scheme(BoxConfig.getInstance().getDownloadUrlScheme());
        builder.encodedAuthority(BoxConfig.getInstance().getDownloadUrlAuthority());
        builder.path(BoxConfig.getInstance().getDownloadUrlPath());
        builder.appendPath(mAuthToken);
        builder.appendPath(fileId);
        return builder.build().toString();
    }

    public Uri registMetaInfo(FileMetaInfo metainf) {
        boolean ret;
        boolean hasRegist = isRegisterdSong(getContext(), metainf.fileId);
        if(!hasRegist){
            Logger.d("metainf.path="+metainf.fileId);
            Logger.d("metainf.name="+metainf.name);
            Logger.d("metainf.title="+metainf.title);
            Logger.d("metainf.album="+metainf.album);
            Logger.d("metainf.artist="+metainf.artist);
            
            //アルバム情報の登録
            ret = isRegisterdAlbum(getContext(), metainf.folderId);
            if(!ret){
                Uri uri = insertAlbum(getContext(), metainf);
                getContext().getContentResolver().notifyChange(MediaConsts.ALBUM_CONTENT_URI, null);
            }
            //アーティスト情報の登録
            if(metainf.artist!=null && metainf.artist.length()>0){
                ret = isRegisterdArtist(getContext(), metainf.artist.hashCode());
                if(!ret){
                    Uri uri = insertArtist(getContext(), metainf);
                    getContext().getContentResolver().notifyChange(MediaConsts.ARTIST_CONTENT_URI, null);
                }
            }
            //楽曲情報の登録
            Uri uri = insertSong(getContext(), metainf);
            getContext().getContentResolver().notifyChange(MediaConsts.MEDIA_CONTENT_URI, null);
        }
        return null;
    }

    public boolean isRegisterdAlbum(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.ALBUM_CONTENT_URI,
                    new String[]{BaseColumns._ID, AudioAlbum.ALBUM}, 
                    AudioAlbum.ALBUM_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public boolean isRegisterdArtist(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.ARTIST_CONTENT_URI, 
                    new String[]{BaseColumns._ID, AudioArtist.ARTIST}, 
                    AudioArtist.ARTIST_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public boolean isRegisterdSong(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.MEDIA_CONTENT_URI, 
                    new String[]{AudioMedia.MEDIA_KEY}, 
                    AudioMedia.MEDIA_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }

    public Uri insertAlbum(Context context, FileMetaInfo metainf){
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            values.put(AudioAlbum.ALBUM_KEY, metainf.folderId);
            values.put(AudioAlbum.ALBUM, metainf.album);
            values.put(AudioAlbum.ALBUM_ART, metainf.albumArt);
            values.put(AudioAlbum.ARTIST, metainf.artist);
            values.put(AudioAlbum.FIRST_YEAR, "");
            values.put(AudioAlbum.LAST_YEAR, "");
            values.put(AudioAlbum.NUMBER_OF_SONGS, "0");
            values.put(AudioAlbum.DATE_ADDED, cal.getTimeInMillis());
            values.put(AudioAlbum.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(TableConsts.ALBUM_DEL_FLG, 0);
            
            return context.getContentResolver().insert(MediaConsts.ALBUM_CONTENT_URI, values);
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public Uri insertArtist(Context context, FileMetaInfo metainf){
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            values.put(AudioArtist.ARTIST_KEY, metainf.artist.hashCode());
            values.put(AudioArtist.ARTIST, metainf.artist);
            values.put(AudioArtist.NUMBER_OF_ALBUMS, "0");
            values.put(AudioArtist.NUMBER_OF_TRACKS, "0");
            values.put(AudioArtist.DATE_ADDED, cal.getTimeInMillis());
            values.put(AudioArtist.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(TableConsts.ARTIST_DEL_FLG, 0);
            
            return context.getContentResolver().insert(MediaConsts.ARTIST_CONTENT_URI, values);
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public Uri insertSong(Context context, FileMetaInfo metainf) {
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            
            values.put(AudioMedia.MEDIA_KEY, metainf.fileId);
            values.put(AudioMedia.TITLE, metainf.title);
            values.put(AudioMedia.TITLE_KEY, metainf.title.hashCode());
            values.put(AudioMedia.DURATION, metainf.duration);
            if(metainf.artist!=null && metainf.artist.length()>0){
                values.put(AudioMedia.ARTIST, metainf.artist);
                values.put(AudioMedia.ARTIST_KEY, metainf.artist.hashCode());
            }
            if(metainf.album!=null && metainf.album.length()>0){
                values.put(AudioMedia.ALBUM, metainf.album);
            }
            else{
                values.put(AudioMedia.ALBUM, "unknown");
            }
            values.put(AudioMedia.ALBUM_KEY, metainf.folderId);
            //values.put(AudioMedia.ALBUM_ART, "");
            values.put(AudioMedia.DATA, metainf.fileId);
            if(metainf.track == null || metainf.track.length() == 0){
                values.put(AudioMedia.TRACK, "9999");
            }
            else{
                values.put(AudioMedia.TRACK, metainf.track);
            }
            values.put(AudioMedia.DATE_ADDED, cal.getTimeInMillis());
            values.put(AudioMedia.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(AudioMedia.YEAR, metainf.year);
            if(metainf.cacheFile!=null){
                values.put(TableConsts.AUDIO_CACHE_FILE, metainf.cacheFile);
            }
            
            Uri uri = context.getContentResolver().insert(MediaConsts.MEDIA_CONTENT_URI, values);
            
            return uri;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
}
